package jeliot.parser;

/**
  * PModifiers class encapsulates the modifiers given to a class,
  * interface or method.
  *
  * @author Pekka Uronen
  *
  * created         5.8.1999
  * last modified   6.9.1999
  */
public class PModifiers {

    public static final int PACKAGE     = 0;
    public static final int PRIVATE     = 1;
    public static final int PROTECTED   = 2;
    public static final int PUBLIC      = 3;

    public static final int STATIC      = 4;
    public static final int ABSTRACT    = 5;
    public static final int FINAL       = 6;
    public static final int NATIVE      = 7;
    public static final int SYNCHRONIZED= 8;
    public static final int TRANSIENT   = 9;
    public static final int VOLATILE    = 10;

    public static final String[] modifierName =
        {   null, "private", "protected", "public",
            "static", "abstract", "final", "synchronized",
            "transient", "volatile" };

    /** Access modifier (one of private, protected, etc.) */
    protected int access = PACKAGE;

    /** Other modifiers (static, abstract, etc.) */
    protected boolean[] modifier = new boolean[VOLATILE - STATIC + 1];

    /** Start positions of the modifiers. left[0] is reserved
      * for the access modifier. */
    protected int[] mLeft = new int[VOLATILE - STATIC];

    /** Start position of this group of modifiers */
    protected int left = Integer.MAX_VALUE;
    
    /** End position of this group of modifiers */
    protected int right = Integer.MIN_VALUE;
    
    /** Creates a new group of modifiers with no members.
      */
    public PModifiers() throws SemanticException { }

    /** Creates a new group of modifiers with the specified modifier as
      * its only member.
      */
    public PModifiers(int m, int left, int right) throws SemanticException {
        setModifier(m, left, right);
    }

    /** Creates a new group of modifiers with the specified modifier as
      * its only member.
      */
    public PModifiers(Integer m, int left, int right) throws SemanticException {
        setModifier(m.intValue(), left, right);
    }

    public void setModifier(Integer m, int left, int right) throws SemanticException {
        setModifier(m.intValue(), left, right);
    }

    /** Sets the modifier given as argument on.
      */
    public void setModifier(int m, int left, int right) throws SemanticException {
        if (m < 4) {
            if (access == 0) {
                access = m;
                mLeft[0] = left;
            }
            else if (access == m) {
                throw new SemanticException("Modifier declared twice: "
                        + modifierName[m]);
            }
            else {
                throw new SemanticException("Conflicting modifiers: "
                        + modifierName[access] + ", " + modifierName[m]);
            }
        }
        else {
            // calculate the index of modifier array
            int i = m - STATIC;
            mLeft[i + 1] = left;
            
            if (modifier[i]) {
                throw new SemanticException("Modifier declared twice: "
                        + modifierName[m]);
            }
            else {
                modifier[i] = true;
            }
        }
        if (this.left > left) {
            this.left = left;
        }
        if (this.right < right) {
            this.right = right;
        }
    }

    /** Returns true, if this pmodifiers instance has given
      * modifier, and false othervise.
      */
    public boolean hasModifier(int m) {
        if (m < 4) {
            return (access == m);
        }
        else {
            // calculate the index of modifier array
            int i = m - STATIC;
            return modifier[i];
        }
    }

    /** Returns the access modifier of this pmodifiers instance.
      */
    public int getAccessModifier() {
        return access;
    }

    public String toString() {
        StringBuffer buff = new StringBuffer();
        String space = "";
        if (access > 0) {
            buff.append(modifierName[access]);
            space = " ";
        }
        
        for (int i = 0; i < modifier.length; ++i) {
            if (modifier[i]) {
                int m = i + STATIC;
                buff.append(space);
                buff.append(modifierName[m]);
                space = " ";
            }
        }
        return buff.toString();
    }

    public int getLeftPos() {
        return left;
    }
}       
